import os
import sys

import sphinx_rtd_theme

sys.path.insert(0, os.path.abspath('..'))

import brreg  # noqa


# -- General configuration ------------------------------------------------

# needs_sphinx = '1.0'

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.viewcode']

templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

project = 'brreg'
copyright = '2019, Otovo AS'
author = 'Otovo AS'

release = brreg.__version__
version = '.'.join(release.split('.')[:2])

language = None
exclude_patterns = ['_build']

pygments_style = 'sphinx'

todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

html_theme = 'sphinx_rtd_theme'
html_theme_options = {}
html_theme_options['canonical_url'] = 'https://brreg.readthedocs.io/en/latest/'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

htmlhelp_basename = 'brreg'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    'papersize': 'a4paper',
    # 'pointsize': '10pt',
    # 'preamble': '',
    # 'figure_align': 'htbp',
}

latex_documents = [
    (master_doc, 'brreg.tex', 'brreg documentation', 'Otovo AS', 'manual')
]


# -- Options for doctest extension ----------------------------------------

autodoc_member_order = 'groupwise'


# -- Options for doctest builder ------------------------------------------

doctest_path = [os.path.abspath('..')]

doctest_global_setup = """
from pprint import pprint
"""

doctest_test_doctest_blocks = 'default'



from typing import Optional

import requests

from brreg import BrregException, BrregRestException
from brreg.enhetsregisteret.types import Enhet


__all__ = ['get_enhet']


BASE_URL = 'https://data.brreg.no/enhetsregisteret/api'


def get_enhet(organisasjonsnummer: str) -> Optional[Enhet]:
    """Get :class:`Enhet` given an organization number.

    Returns :class:`None` if Enhet is gone or not found
    Returns :class:`Enhet` if Enhet is found

    Raises :class:`BrregRestException` if a REST exception occures
    Raises :class:`BrregException` if an unhandled exception occures
    """

    try:
        res = requests.get(f'{BASE_URL}/enheter/{organisasjonsnummer}')

        if res.status_code in (404, 410):
            return None

        res.raise_for_status()

        return Enhet.from_json(res.json())
    except requests.RequestException as exc:
        raise BrregRestException(
            exc,
            method=exc.request.method,
            url=exc.request.url,
            status=getattr(exc.response, 'status_code', None),
        ) from exc
    except Exception as exc:
        raise BrregException(exc)



from pathlib import Path

import pytest


TEST_DIR = Path(__file__).parent


@pytest.fixture
def organization_details_response():
    filepath = TEST_DIR / 'data' / 'organization-details-response.json'
    return filepath.read_bytes()


@pytest.fixture
def deleted_organization_details_response():
    filepath = TEST_DIR / 'data' / 'deleted-organization-details-response.json'
    return filepath.read_bytes()



from datetime import date

import pytest

import responses

from brreg import BrregRestException
from brreg import enhetsregisteret


@responses.activate
def test_get_enhet(organization_details_response):
    responses.add(
        responses.GET,
        'https://data.brreg.no/enhetsregisteret/api/enheter/818511752',
        body=organization_details_response,
        status=200,
        content_type='application/json',
    )

    org = enhetsregisteret.get_enhet('818511752')

    assert org.organisasjonsnummer == '818511752'
    assert org.navn == 'SESAM STASJON'
    assert org.hjemmeside is None
    assert org.registreringsdato_enhetsregisteret == date(2017, 10, 20)
    assert org.registrert_i_mvaregisteret is True
    assert org.naeringskode1 == enhetsregisteret.Naeringskode(
        kode='52.292', beskrivelse='Skipsmegling'
    )
    assert org.antall_ansatte == 50
    assert org.forretningsadresse == enhetsregisteret.Adresse(
        land='Norge',
        landkode='NO',
        postnummer='0101',
        poststed='OSLO',
        adresse=['Tyvholmen 1', None, None, ''],
        kommune='OSLO',
        kommunenummer='0301',
    )
    assert org.stiftelsesdato == date(2017, 10, 20)
    assert org.institusjonell_sektorkode is None
    assert org.registrert_i_foretaksregisteret is True
    assert org.registrert_i_stiftelsesregisteret is False
    assert org.registrert_i_frivillighetsregisteret is False
    assert org.siste_innsendte_aarsregnskap is None
    assert org.konkurs is False
    assert org.under_avvikling is False
    assert org.under_tvangsavvikling_eller_tvangsopplosning is False
    assert org.maalform == 'Bokmål'
    assert org.slettedato is None


@responses.activate
def test_get_enhet_when_deleted(deleted_organization_details_response):
    responses.add(
        responses.GET,
        'https://data.brreg.no/enhetsregisteret/api/enheter/815597222',
        body=deleted_organization_details_response,
        status=200,
        content_type='application/json',
    )

    org = enhetsregisteret.get_enhet('815597222')

    assert org.organisasjonsnummer == '815597222'
    assert org.navn == 'SLETTET ENHET AS'
    assert org.organisasjonsform == enhetsregisteret.Organisasjonsform(
        kode='UTBG', beskrivelse='Frivillig registrert utleiebygg'
    )
    assert org.slettedato == date(2017, 10, 20)


@responses.activate
def test_get_enhet_when_gone():
    responses.add(
        responses.GET,
        'https://data.brreg.no/enhetsregisteret/api/enheter/818511752',
        status=410,
        content_type='application/json',
    )

    org = enhetsregisteret.get_enhet('818511752')

    assert org is None


@responses.activate
def test_get_enhet_when_not_found():
    responses.add(
        responses.GET,
        'https://data.brreg.no/enhetsregisteret/api/enheter/818511752',
        status=404,
        content_type='application/json',
    )

    org = enhetsregisteret.get_enhet('818511752')

    assert org is None


@responses.activate
def test_get_enhet_when_http_error():
    responses.add(
        responses.GET,
        'https://data.brreg.no/enhetsregisteret/api/enheter/818511752',
        status=400,
        content_type='application/json',
    )

    with pytest.raises(BrregRestException) as exc_info:
        enhetsregisteret.get_enhet('818511752')

    assert 'REST API exception' in str(exc_info.value)
    assert 'Bad Request' in str(exc_info.value)

    assert exc_info.value.method == 'GET'
    assert (
        exc_info.value.url
        == 'https://data.brreg.no/enhetsregisteret/api/enheter/818511752'
    )
    assert exc_info.value.status == 400


@responses.activate
def test_get_organization_by_number_when_http_timeout():
    with pytest.raises(BrregRestException) as exc_info:
        enhetsregisteret.get_enhet('818511752')

    assert 'REST API exception' in str(exc_info.value)
    assert 'Connection refused' in str(exc_info.value)

    assert exc_info.value.method == 'GET'
    assert (
        exc_info.value.url
        == 'https://data.brreg.no/enhetsregisteret/api/enheter/818511752'
    )
    assert exc_info.value.status is None



from datetime import date, datetime
from typing import List, Optional

import attr


__all__ = [
    'Adresse',
    'Enhet',
    'InstitusjonellSektorkode',
    'Naeringskode',
    'Organisasjonsform',
]


@attr.s(auto_attribs=True)
class InstitusjonellSektorkode:

    #: Sektorkoden
    kode: str

    #: Tekstlig beskrivelse av sektorkoden
    beskrivelse: str

    def __str__(self):
        return f'{self.beskrivelse} ({self.kode})'

    @classmethod
    def from_json(
        cls, json: Optional[dict]
    ) -> Optional['InstitusjonellSektorkode']:
        if not json:
            return None

        return cls(kode=json['kode'], beskrivelse=json['beskrivelse'])


@attr.s(auto_attribs=True)
class Adresse:
    #: Land
    land: str

    #: Landkode
    landkode: str

    #: Postnummer
    postnummer: str

    #: Poststed
    poststed: str

    #: Adresse
    adresse: List[str]

    #: Kommune
    kommune: str

    #: Kommunenummer
    kommunenummer: str

    def __str__(self):
        return self.adresse[0]

    @classmethod
    def from_json(cls, json: Optional[dict]) -> Optional['Adresse']:
        if not json:
            return None

        return cls(
            land=json['land'],
            landkode=json['landkode'],
            postnummer=json['postnummer'],
            poststed=json['poststed'],
            adresse=json['adresse'],
            kommune=json['kommune'],
            kommunenummer=json['kommunenummer'],
        )


@attr.s(auto_attribs=True)
class Naeringskode:

    #: Næringskoden
    kode: str

    #: Tekstlig beskrivelse av næringskoden
    beskrivelse: str

    def __str__(self):
        return f'{self.beskrivelse} ({self.kode})'

    @classmethod
    def from_json(cls, json: Optional[dict]) -> Optional['Naeringskode']:
        if not json:
            return None
        return cls(kode=json['kode'], beskrivelse=json['beskrivelse'])


@attr.s(auto_attribs=True)
class Organisasjonsform:

    #: Organisasjonsformen
    kode: str

    #: Tekstlig beskrivelse av organisasjonsformen
    beskrivelse: str

    def __str__(self):
        return f'{self.beskrivelse} ({self.kode})'

    @classmethod
    def from_json(cls, json: dict) -> 'Organisasjonsform':
        return cls(kode=json['kode'], beskrivelse=json['beskrivelse'])


@attr.s(auto_attribs=True)
class Enhet:
    #: Organisasjonsnummer
    organisasjonsnummer: str

    #: Navn
    navn: str

    #: Organisasjonsform
    organisasjonsform: Organisasjonsform

    #: Hjemmeside
    hjemmeside: Optional[str]

    #: Registreringsdato i Enhetsregisteret
    registreringsdato_enhetsregisteret: Optional[date]

    #: Hvorvidt enheten er registrert i MVA-registeret
    registrert_i_mvaregisteret: Optional[bool]

    #: Næringskode 1
    naeringskode1: Optional[Naeringskode]

    #: Antall ansatte
    antall_ansatte: Optional[int]

    #: Forretningsadresse
    forretningsadresse: Optional[Adresse]

    #: Stiftelsesdato
    stiftelsesdato: Optional[date]

    #: Sektorkode
    institusjonell_sektorkode: Optional[InstitusjonellSektorkode]

    #: Hvorvidt enheten er registrert i Foretaksregisteret
    registrert_i_foretaksregisteret: Optional[bool]

    #: Hvorvidt enheten er registrert i Stiftelsesregisteret
    registrert_i_stiftelsesregisteret: Optional[bool]

    #: Hvorvidt enheten er registrert i Frivillighetsregisteret
    registrert_i_frivillighetsregisteret: Optional[bool]

    #: År for siste innsendte årsregnskap
    siste_innsendte_aarsregnskap: Optional[int]

    #: Hvorvidt enheten er konkurs
    konkurs: Optional[bool]

    #: Hvorvidt enheten er under avvikling
    under_avvikling: Optional[bool]

    #: Hvorvidt enheten er under tvangsavvikling eller tvangsoppløsning
    under_tvangsavvikling_eller_tvangsopplosning: Optional[bool]

    #: Målform
    maalform: Optional[str]

    #: Dato enheten ble slettet
    slettedato: Optional[date]

    def __str__(self):
        return f'{self.navn} ({self.organisasjonsnummer})'

    @classmethod
    def from_json(cls, json: dict) -> Optional['Enhet']:
        if not json:
            return None

        return cls(
            organisasjonsnummer=json['organisasjonsnummer'],
            navn=json['navn'],
            organisasjonsform=Organisasjonsform.from_json(
                json['organisasjonsform']
            ),
            hjemmeside=json.get('hjemmeside'),
            registreringsdato_enhetsregisteret=parse_date(
                json.get('registreringsdatoEnhetsregisteret')
            ),
            registrert_i_mvaregisteret=json.get('registrertIMvaregisteret'),
            naeringskode1=Naeringskode.from_json(json.get('naeringskode1')),
            antall_ansatte=json.get('antallAnsatte'),
            forretningsadresse=Adresse.from_json(
                json.get('forretningsadresse')
            ),
            stiftelsesdato=parse_date(json.get('stiftelsesdato')),
            institusjonell_sektorkode=InstitusjonellSektorkode.from_json(
                json.get('institusjonellSektorkode')
            ),
            registrert_i_foretaksregisteret=json.get(
                'registrertIForetaksregisteret'
            ),
            registrert_i_stiftelsesregisteret=json.get(
                'registrertIStiftelsesregisteret'
            ),
            registrert_i_frivillighetsregisteret=json.get(
                'registrertIFrivillighetsregisteret'
            ),
            siste_innsendte_aarsregnskap=parse_int(
                json.get('sisteInnsendteAarsregnskap')
            ),
            konkurs=json.get('konkurs'),
            under_avvikling=json.get('underAvvikling'),
            under_tvangsavvikling_eller_tvangsopplosning=json.get(
                'underTvangsavviklingEllerTvangsopplosning'
            ),
            maalform=json.get('maalform'),
            slettedato=parse_date(json.get('slettedato')),
        )


def parse_date(date_string: Optional[str]) -> Optional[date]:
    if date_string is None:
        return None
    return datetime.strptime(date_string, '%Y-%m-%d').date()


def parse_int(int_string: Optional[str]) -> Optional[int]:
    if int_string is None:
        return None
    return int(int_string)



"""
API client for `Enhetsregisteret <https://w2.brreg.no/enhet/sok/>`_.

See https://data.brreg.no/enhetsregisteret/api/docs/index.html for API details.
"""


from brreg.enhetsregisteret.types import *  # noqa: Reexport
from brreg.enhetsregisteret.client import *  # noqa: Reexport


from brreg.enhetsregisteret import (  # noqa: Must come after reexport
    client,
    types,
)


__all__ = client.__all__ + types.__all__



import re

from setuptools import find_packages, setup


with open('brreg/__init__.py') as fh:
    metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", fh.read()))


with open('README.rst') as fh:
    long_description = fh.read()


setup(
    name='brreg',
    version=metadata['version'],
    description='API client for Brønnøysundregistrene',
    long_description=long_description,
    url='https://brreg.readthedocs.io/',
    author='Otovo AS',
    author_email='andrroy+brreg@otovo.com',
    license='Apache License, Version 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='brreg enhetsregisteret',
    packages=find_packages(exclude=['tests', 'tests.*']),
    python_requires='>=3.6',
    install_requires=['attrs >= 17.4', 'requests'],
    extras_require={
        'dev': [
            'black',
            'check-manifest',
            'flake8',
            'flake8-bugbear',
            'flake8-comprehensions',
            'flake8-import-order',
            'flake8-quotes',
            'flake8-tidy-imports',
            'pep8-naming',
            'mypy',
            'pydocstyle',
            'pytest',
            'pytest-xdist',
            'responses',
            'sphinx',
            'sphinx_rtd_theme',
            'tox',
        ]
    },
)



__all__ = ['BrregException', 'BrregRestException']


class BrregException(Exception):
    """Top-level exception.

    All exceptions raised by the :mod:`brreg` library are subclasses of this
    exception.
    """

    pass


class BrregRestException(BrregException):
    """REST API exception."""

    def __init__(self, msg, *, method, url, status):
        super().__init__(f'REST API exception: {msg}')
        self.method = method
        self.url = url
        self.status = status
